#!/bin/sh
unset LD_LIBRARY_PATH
VERSION=v2
CTNG_VERSION=1.13.2
TARGETS="\
	powerpc-750-linux-gnu \
	powerpc-7450-linux-gnu \
	powerpc-440fp-linux-gnu \
	arm-926ejs-linux-gnueabi \
"

NUM_PROCESSORS=`cat /proc/cpuinfo | cut -f 1 | grep vendor_id | wc -l`
HERE=`pwd`
MY_DIR=`dirname $0`
if [ ${MY_DIR} = "." ]; then
	MY_DIR=${HERE}
elif [ ${MY_DIR} = ".." ]; then
	MY_DIR=${HERE}/..
fi
TARBALLS_DIR=${MY_DIR}/downloads

Usage ()
{
	echo "Usage:"
	echo "  $0 all <target>          build everything"
	echo "  $0 clean <target>        clean everything"
	echo "  dist-source  tar all sources in a unique tarball"
}

CMD="$1"
case "${CMD}" in
	all | clean)
		TARGET="$2"
		if [ -z "${TARGET}" ]; then
			Usage
			exit -1
		fi
		SUPPORTED_TARGET=no
		for TGT in ${TARGETS}; do
			if [ "${TARGET}" = "${TGT}" ]; then
				SUPPORTED_TARGET=yes
			fi
		done
		if [ "${SUPPORTED_TARGET}" = "no" ]; then
			echo "Error: unsupported Target '${TARGET}'"
			Usage
			exit -1
		fi
		;;
	dist-source)
		FILE_LIST="\
		downloads/linux-3.0.4.tar.xz \
		downloads/mpfr-3.0.0.tar.xz \
		downloads/gcc-4.4.6.tar.bz2 \
		downloads/crosstool-ng-${CTNG_VERSION}.tar.bz2 \
		downloads/ppl-0.10.2.tar.bz2 \
		downloads/glibc-2.9.tar.bz2 \
		downloads/gmp-4.3.2.tar.bz2 \
		downloads/glibc-ports-2.9.tar.bz2 \
		downloads/gdb-6.8a.tar.bz2 \
		downloads/cloog-ppl-0.15.10.tar.gz \
		downloads/binutils-2.21.1a.tar.bz2 \
		$(basename $0) \
		"

		for TARGET in ${TARGETS}; do
			FILE_LIST="${FILE_LIST} config/crosstool-ng-${CTNG_VERSION}-${TARGET}-config"
		done

		PKG_NAME="crosstool-source-${VERSION}"

		rm -rf ${HERE}/${PKG_NAME}
		rm -f ${HERE}/${PKG_NAME}.tar.gz
		mkdir -p ${HERE}/${PKG_NAME}

		for FILE in ${FILE_LIST}; do
			mkdir -p ${HERE}/${PKG_NAME}/$(dirname ${FILE})
			cp ${MY_DIR}/${FILE} ${HERE}/${PKG_NAME}/${FILE} || exit -1
		done

		cat << EOF > "${HERE}/${PKG_NAME}/README"
This package contains everything to automatically build a crosstool chain for targets ${TARGETS}.
Run the script named build.sh to start playing with it.

This package contains:
- the source tarball of Crosstool-NG ${CTNG_VERSION}
- all the source tarballs that Crosstool-NG ${CTNG_VERSION} needs to build the crosstool chain
- the configuration file of Crosstool-NG ${CTNG_VERSION} to build a crosstool chain for targets ${TARGETS}
- the building script build.sh
EOF
		cd ${HERE}
		tar zcvf ${HERE}/${PKG_NAME}.tar.gz ${PKG_NAME}
		rm -rf ${HERE}/${PKG_NAME}
		;;
	*)
		echo "FATAL: Please specify a command"
		Usage
		exit 255
		;;
esac

CTNG_CONFIG="${MY_DIR}/config/crosstool-ng-${CTNG_VERSION}-${TARGET}-config"

case "${CMD}" in
	all)
		if [ ! -e ${TARBALLS_DIR}/crosstool-ng-${CTNG_VERSION}.tar.bz2 ]; then
			echo "Downloading crosstool-ng source tarball. Please wait..."
			wget http://crosstool-ng.org/download/crosstool-ng/crosstool-ng-${CTNG_VERSION}.tar.bz2 -P ${TARBALLS_DIR}
			if [ $? -ne 0 ]; then
				echo "Downloading crosstool-ng source tarball failed"
				exit -1
			fi
		fi

		if [ ! -d ${MY_DIR}/crosstool-ng-${CTNG_VERSION} ]; then
			echo "Uncompressing crosstool-ng source tarball. Please wait..."
			tar jxvf ${TARBALLS_DIR}/crosstool-ng-${CTNG_VERSION}.tar.bz2
			cd ${MY_DIR}
		fi

		if [ ! -e ${MY_DIR}/crosstool-ng-${CTNG_VERSION}/ct-ng ]; then
			echo "Building crosstool-ng (ct-ng). Please wait..."
			cd ${MY_DIR}/crosstool-ng-${CTNG_VERSION}
			./configure --local || exit 255
			make
			if [ $? -ne 0 ]; then
				echo "Building crosstool-ng (ct-ng) failed"
				exit -1
			fi
			sed -i -e '/--disable-debug/d' "${MY_DIR}/crosstool-ng-${CTNG_VERSION}/scripts/build/libc/glibc.sh"
			sed -i -e 's/--disable-debug//g' "${MY_DIR}/crosstool-ng-${CTNG_VERSION}/scripts/build/libc/glibc-eglibc.sh-common"
		fi
		
		echo "Build GCC cross-compiler for ${TARGET}. Please wait..."
		cat ${CTNG_CONFIG} | \
			sed -e "s#\(CT_PREFIX_DIR=\).*\$#\1\"${MY_DIR}/${TARGET}\"#" \
				-e "s#\(CT_LOCAL_TARBALLS_DIR=\).*\$#\1\"${MY_DIR}/downloads\"#" \
				-e "s#\(CT_PARALLEL_JOBS=\).*\$#\1${NUM_PROCESSORS}#" \
				> "${MY_DIR}/crosstool-ng-${CTNG_VERSION}/.config"

		cd "${MY_DIR}/crosstool-ng-${CTNG_VERSION}"
		./ct-ng build
		if [ $? -ne 0 ]; then
			echo "Building GCC cross-compiler for ${TARGET} failed"
			exit -1
		fi
		echo "GCC cross-compiler for ${TARGET} is in ${MY_DIR}/${TARGET}"
		;;
	clean)
		echo "Cleaning crosstool-ng and GCC cross-compiler for ${TARGET}. Please wait..."
		rm -rf "${MY_DIR}/crosstool-ng-${CTNG_VERSION}"
		if [ -d "${MY_DIR}/${TARGET}" ]; then
			chmod -R +w "${MY_DIR}/${TARGET}"
		fi
		rm -rf "${MY_DIR}/${TARGET}"
		echo "Your build has been cleaned"
		;;
esac


